/************************************************************************
	interrupt.c

    Simon18F - An MB Electronics Simon clone based on the PIC18F2550
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#include <htc.h>

#include "interrupt.h"
#include "hardware.h"

// Low priority interrupt procedure
void interrupt low_priority lpHandler(void)
{

	// Is this timer0 interrupting?
	if (TMR0IF)
	{
		// Write the state of the lights **********************************************************

		// Control the blue light:
		if (hardware.light[BLUE].brightness == 100)
		{
			BLUE_LIGHT = 1;
		}
		else
		{
			if (hardware.light[BLUE].brightness == 0)
			{
				BLUE_LIGHT = 0;
			}
			else
			{
				if (hardware.pwmCounter < hardware.light[BLUE].brightness)
					BLUE_LIGHT = 1;
				else BLUE_LIGHT = 0;
			}
		}
	
		// If light is OFF and brightness > 0 then fade the light at dimSpeed
		if ((hardware.light[BLUE].state == OFF) && (hardware.light[BLUE].brightness > 0))
		{
			if (hardware.dimCounter == 0)
			{
				hardware.light[BLUE].brightness -= hardware.light[BLUE].dimSpeed;
				if (hardware.light[BLUE].brightness < 0) hardware.light[BLUE].brightness = 0;
			}
		}

		// Control the red light:
		if (hardware.light[RED].brightness == 100)
		{
			RED_LIGHT = 1;
		}
		else
		{
			if (hardware.light[RED].brightness == 0)
			{
				RED_LIGHT = 0;
			}
			else
			{
				if (hardware.pwmCounter < hardware.light[RED].brightness)
					RED_LIGHT = 1;
				else RED_LIGHT = 0;
			}
		}
	
		// If light is OFF and brightness > 0 then fade the light at dimSpeed
		if ((hardware.light[RED].state == OFF) && (hardware.light[RED].brightness > 0))
		{
			if (hardware.dimCounter == 0)
			{
				hardware.light[RED].brightness -= hardware.light[RED].dimSpeed;
				if (hardware.light[RED].brightness < 0) hardware.light[RED].brightness = 0;
			}
		}
	
		// Control the green light:
		if (hardware.light[GREEN].brightness == 100)
		{
			GREEN_LIGHT = 1;
		}
		else
		{
			if (hardware.light[GREEN].brightness == 0)
			{
				GREEN_LIGHT = 0;
			}
			else
			{
				if (hardware.pwmCounter < hardware.light[GREEN].brightness)
					GREEN_LIGHT = 1;
				else GREEN_LIGHT = 0;
			}
		}
	
		// If light is OFF and brightness > 0 then fade the light at dimSpeed
		if ((hardware.light[GREEN].state == OFF) && (hardware.light[GREEN].brightness > 0))
		{
			if (hardware.dimCounter == 0)
			{
				hardware.light[GREEN].brightness -= hardware.light[GREEN].dimSpeed;
				if (hardware.light[GREEN].brightness < 0) hardware.light[GREEN].brightness = 0;
			}
		}
	
		// Control the yellow light:
		if (hardware.light[YELLOW].brightness == 100)
		{
			YELLOW_LIGHT = 1;
		}
		else
		{
			if (hardware.light[YELLOW].brightness == 0)
			{
				YELLOW_LIGHT = 0;
			}
			else
			{
				if (hardware.pwmCounter < hardware.light[YELLOW].brightness)
					YELLOW_LIGHT = 1;
				else YELLOW_LIGHT = 0;
			}
		}
	
		// If light is OFF and brightness > 0 then fade the light at dimSpeed
		if ((hardware.light[YELLOW].state == OFF) && (hardware.light[YELLOW].brightness > 0))
		{
			if (hardware.dimCounter == 0)
			{
				hardware.light[YELLOW].brightness -= hardware.light[YELLOW].dimSpeed;
				if (hardware.light[YELLOW].brightness < 0) hardware.light[YELLOW].brightness = 0;
			}
		}

		// Increment the PWM counter
		hardware.pwmCounter++;
		if (hardware.pwmCounter == 100) hardware.pwmCounter = 0;

		// Increment the dimming counter
		hardware.dimCounter++;
		if (hardware.dimCounter == 200) hardware.dimCounter = 0;

		// Read the state of the buttons **********************************************************
		if (hardware.buttonCounter == 0)
		{
			// Read the state of the start button and skill level slider
			if (IN1 == ON || IN2 == ON || IN3 == ON || IN4 == ON)
			{
				if (hardware.startButton.state == OFF)
				{
					// If the debounce tolerance is met change state otherwise
					// increment the debounce counter
					if (hardware.startButton.debounceCount > DEBOUNCE_TOLERANCE)
					{
						// Start button is on
						hardware.startButton.state = ON;

						// Now determine the skill level setting
						if(IN1 == ON) hardware.skillSwitch.position = 1;
						if(IN3 == ON) hardware.skillSwitch.position = 2;
						if(IN2 == ON) hardware.skillSwitch.position = 3;
						if(IN4 == ON) hardware.skillSwitch.position = 4;
					}
					else hardware.startButton.debounceCount++;
				}
			}
			else
			{
				// Button is off
				hardware.startButton.state = OFF;
				hardware.startButton.debounceCount = 0;
				//hardware.startButton.autoOffCount = 0;
			}

			// Increment the random number counter
			if (game.randomNumber == 3) game.randomNumber = 0;
			else game.randomNumber++;

			// Set the ports ready for the next read
			START_SKILL = OFF;
			LAST_LONGEST_GAME = ON;
			COLOUR_BUTTONS = OFF;
		}

		if (hardware.buttonCounter == 30)
		{
			// Read the state of the last button, longest button and the game slider

			// NOTE: This code is not good; since we will want to keep the light on
			//   whilst the button is pressed we should debounce both the ON transisition
			//   and the OFF transistion.

			// Last
			if (IN4 == ON)
			{
				if (hardware.lastButton.state == OFF)
				{
					// If the debounce tolerance is met change state otherwise
					// increment the debounce counter
					if (hardware.lastButton.debounceCount > DEBOUNCE_TOLERANCE)
						hardware.lastButton.state = ON;
					else hardware.lastButton.debounceCount++;
				}
			}
			else
			{
				// Button is off
				hardware.lastButton.state = OFF;
				hardware.lastButton.debounceCount = 0;
			}
		
			// Longest
			if (IN3 == ON)
			{
				if (hardware.longestButton.state == OFF)
				{
					// If the debounce tolerance is met change state otherwise
					// increment the debounce counter
					if (hardware.longestButton.debounceCount > DEBOUNCE_TOLERANCE)
						hardware.longestButton.state = ON;
					else hardware.longestButton.debounceCount++;
				}
			}
			else
			{
				// Button is off
				hardware.longestButton.state = OFF;
				hardware.longestButton.debounceCount = 0;
			}
		
			// Game
			if (IN1 == OFF && IN2 == ON ) hardware.gameSwitch.position = 3;
			else if (IN1 == ON  && IN2 == OFF) hardware.gameSwitch.position = 2;
			else hardware.gameSwitch.position = 1;
			

			// Set the ports ready for the next read
			START_SKILL = OFF;
			LAST_LONGEST_GAME = OFF;
			COLOUR_BUTTONS = ON;
		}

		if (hardware.buttonCounter == 60)
		{
			// Read the state of the colour buttons
			unsigned char button;
			unsigned char input;

			for (button = 0; button < 4; button++)
			{
				switch (button)
				{
					case 0:	input = IN1;	// Blue
							break;
					case 1: input = IN2;	// Yellow
							break;
					case 2:	input = IN3;	// Red
							break;
					case 3:	input = IN4;	// Green
							break;
				}

				// Debounce ON
				if (input == ON && hardware.colourButton[button].state == OFF)
				{
					// If the debounce tolerance is met change state otherwise
					// increment the debounce counter
					if (hardware.colourButton[button].debounceCount > DEBOUNCE_TOLERANCE)
						hardware.colourButton[button].state = ON;
					else hardware.colourButton[button].debounceCount++;
				}
	
				// Debounce off
				if (input == OFF && hardware.colourButton[button].state == ON)
				{
					// If the debounce tolerance is met change state otherwise
					// increment the debounce counter
					if (hardware.colourButton[button].debounceCount > DEBOUNCE_TOLERANCE)
						hardware.colourButton[button].state = OFF;
					else hardware.colourButton[button].debounceCount++;
				}
	
				// Reset debounce counter
				if (input == hardware.colourButton[button].state) hardware.colourButton[button].debounceCount = 0;
			}

			// Set the ports ready for the next read
			START_SKILL = ON;
			LAST_LONGEST_GAME = OFF;
			COLOUR_BUTTONS = OFF;
		}

		// Update the button counter
		hardware.buttonCounter++;
		if (hardware.buttonCounter == 90) hardware.buttonCounter = 0;

		// Increment the delay counter
		hardware.delayCounter++;
		if (hardware.delayCounter > 4000000000) hardware.delayCounter = 0;

		TMR0L = TMR0_SPEED;	// Reset the timer0 counter
		TMR0IF = 0;			// Clear the timer0 interrupt flag
	}
}

// High priority interrupt procedure
void interrupt hpHandler(void)
{
	// Is this timer1 interrupting? (PWM sound generation interrupt)
	if (TMR1IF)
	{
		// Pulse on or pulse off?
		if (hardware.sound.pwmState == OFF)
		{
			SPEAKER = 1;
			hardware.sound.pwmState = ON;
		}
		else
		{
			SPEAKER = 0;
			hardware.sound.pwmState = OFF;
		}

		// Reset the timer1 counter bytes
		TMR1H = hardware.sound.timer1HighByte;
		TMR1L = hardware.sound.timer1LowByte;

		// Reset the timer1 interrupt flag
		TMR1IF = 0;
	}
}
